# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ClusterStartup(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'start_time': 'datetime',
        'cluster_id': 'str',
        'id': 'str',
        'finish_time': 'datetime'
    }

    attribute_map = {
        'start_time': 'start_time',
        'cluster_id': 'cluster_id',
        'id': 'id',
        'finish_time': 'finish_time'
    }

    def __init__(self, start_time=None, cluster_id=None, id=None, finish_time=None, local_vars_configuration=None):  # noqa: E501
        """ClusterStartup - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._start_time = None
        self._cluster_id = None
        self._id = None
        self._finish_time = None
        self.discriminator = None

        self.start_time = start_time
        if cluster_id is not None:
            self.cluster_id = cluster_id
        self.id = id
        if finish_time is not None:
            self.finish_time = finish_time

    @property
    def start_time(self):
        """Gets the start_time of this ClusterStartup.  # noqa: E501

        When the startup was initiated. Should be timezone-aware.  # noqa: E501

        :return: The start_time of this ClusterStartup.  # noqa: E501
        :rtype: datetime
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """Sets the start_time of this ClusterStartup.

        When the startup was initiated. Should be timezone-aware.  # noqa: E501

        :param start_time: The start_time of this ClusterStartup.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and start_time is None:  # noqa: E501
            raise ValueError("Invalid value for `start_time`, must not be `None`")  # noqa: E501

        self._start_time = start_time

    @property
    def cluster_id(self):
        """Gets the cluster_id of this ClusterStartup.  # noqa: E501

        Cluster ID from sessions table. None if not known yet.  # noqa: E501

        :return: The cluster_id of this ClusterStartup.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this ClusterStartup.

        Cluster ID from sessions table. None if not known yet.  # noqa: E501

        :param cluster_id: The cluster_id of this ClusterStartup.  # noqa: E501
        :type: str
        """

        self._cluster_id = cluster_id

    @property
    def id(self):
        """Gets the id of this ClusterStartup.  # noqa: E501

        Unique identifier for this startup.  # noqa: E501

        :return: The id of this ClusterStartup.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ClusterStartup.

        Unique identifier for this startup.  # noqa: E501

        :param id: The id of this ClusterStartup.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def finish_time(self):
        """Gets the finish_time of this ClusterStartup.  # noqa: E501

        When this cluster startup has finished; None if it is still in progress.  # noqa: E501

        :return: The finish_time of this ClusterStartup.  # noqa: E501
        :rtype: datetime
        """
        return self._finish_time

    @finish_time.setter
    def finish_time(self, finish_time):
        """Sets the finish_time of this ClusterStartup.

        When this cluster startup has finished; None if it is still in progress.  # noqa: E501

        :param finish_time: The finish_time of this ClusterStartup.  # noqa: E501
        :type: datetime
        """

        self._finish_time = finish_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ClusterStartup):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ClusterStartup):
            return True

        return self.to_dict() != other.to_dict()
